@@ -11,6 +11,11 @@
 #  include <hildon-widgets/hildon-banner.h>
 #  include <hildon-widgets/hildon-file-chooser-dialog.h>
 #  include <hildon-widgets/hildon-number-editor.h>
+
+HildonNumberEditor* get_ned(const char *spin_name);
+gint get_hildon_value(const char *spin_name);
+void set_hildon_value(const char *spin_name, gint val);
+gint check_name(gconstpointer list_node, gconstpointer name);
 #endif
 
 #include "callbacks.h"
@@ -25,31 +30,28 @@
 void show_error_dialog(GtkWidget *mainwindow, const char *error_msg);
 
 // MAME
-void on_b_ok_clicked                    (GtkButton       *button,
+void on_mame_launch_clicked                    (GtkButton       *button,
                                         gpointer         user_data) {
 	gchar	*romname = NULL,
 		*cursor = NULL;
 	GtkWidget	*w = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist"),
-			*mainwindow = lookup_widget(GTK_WIDGET(button), "mainwindow");
-	const char*	wnames[4] = { "cb_sound", "cb_fullscreen", "cb_ror", "cb_scale" };
+			*romlist = glade_xml_get_widget(glade, "mame_romlist"),
+			*mainwindow = glade_xml_get_widget(glade, "mainwindow");
+	const char*	wnames[4] = { "mame_sound", "mame_fullscreen", "mame_rotate", "mame_double" };
 	gboolean	flags[4] = { TRUE, TRUE, TRUE, TRUE };
 	gint	i = 0,
 		status = 0;
 
 	for(i=0;i<4;++i){
-		w=lookup_widget(GTK_WIDGET(button), wnames[i]);
-		if(w){
-			flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-		}//if
+		w=glade_xml_get_widget(glade, wnames[i]);
+		if(w){ flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)); }
 	}//for
 
 	romname = gtk_combo_box_get_active_text(GTK_COMBO_BOX(romlist));
 	//strip_extension(romname);
 
-
 #ifdef HILDON
-	hildon_banner_show_information(mainwindow, NULL, _("Launching xmame"));
+	hildon_banner_show_information(GTK_WIDGET(hildonwindow), NULL, _("Launching xmame"));
 #endif
 
 	status = launch_xmame(romname, flags[0], flags[1], flags[2], flags[3]);
@@ -66,37 +68,37 @@ void on_b_ok_clicked                    (GtkButton       *button,
 
 
 //NES
-void on_b_ok2_clicked                   (GtkButton       *button,
+void on_nes_launch_clicked                   (GtkButton       *button,
                                         gpointer         user_data) {
 	gchar	*romname = NULL,
-		*cursor = NULL;
+		*wname = NULL;
 	GtkWidget	*w = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist2"),
-			*mainwindow = lookup_widget(GTK_WIDGET(button), "mainwindow");
-	const char*	wnames[4] = { "cb_sound2", "cb_fullscreen2", "cb_scale2", "frameskip" };
+			*romlist = glade_xml_get_widget(glade, "nes_romlist"),
+			*mainwindow = glade_xml_get_widget(glade, "mainwindow");
+	const char*	wnames[4] = { "nes_sound", "nes_fullscreen", "nes_double", "nes_frameskip" };
 	gint	flags[4] = { TRUE, TRUE, TRUE, 2 };
 	gint	i = 0,
 		status = 0;
+	GList	*list = NULL,
+		*cursor = NULL;
 
 	for(i=0;i<3;++i){
-		w=lookup_widget(GTK_WIDGET(button), wnames[i]);
-		if(w){
-			flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-		}//if
+		w=glade_xml_get_widget(glade, wnames[i]);
+		if(w){ flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)); }//if
 	}//for
 
-	w=lookup_widget(GTK_WIDGET(button), wnames[3]);
 
 #ifdef HILDON
-	flags[3] = hildon_number_editor_get_value(HILDON_NUMBER_EDITOR(w));
+	flags[3] = get_hildon_value(wnames[3]);
 #else
+	w=glade_xml_get_widget(glade, wnames[3]);
 	flags[3] = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
 #endif
 
 	romname = gtk_combo_box_get_active_text(GTK_COMBO_BOX(romlist));
 
 #ifdef HILDON
-	hildon_banner_show_information(mainwindow, NULL, _("Launching fceu"));
+	hildon_banner_show_information(GTK_WIDGET(hildonwindow), NULL, _("Launching fceu"));
 #endif
 
 	status = launch_fceu(romname, flags[0], flags[1], flags[2], flags[3]);
@@ -113,36 +115,34 @@ void on_b_ok2_clicked                   (GtkButton       *button,
 
 
 // GBA
-void on_b_ok3_clicked                   (GtkButton       *button,
+void on_gba_launch_clicked                   (GtkButton       *button,
                                         gpointer         user_data) {
 	gchar	*romname = NULL,
 		*cursor = NULL;
 	GtkWidget	*w = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist3"),
-			*mainwindow = lookup_widget(GTK_WIDGET(button), "mainwindow");
-	const char*	wnames[3] = { "cb_fullscreen3", "cb_scale3", "frameskip3" };
+			*romlist = glade_xml_get_widget(glade, "gba_romlist"),
+			*mainwindow = glade_xml_get_widget(glade, "mainwindow");
+	const char*	wnames[3] = { "gba_fullscreen", "gba_double", "gba_frameskip" };
 	gint	flags[3] = { TRUE, TRUE, 5 };
 	gint	i = 0,
 		status = 0;
 
 	for(i=0;i<2;++i){
-		w=lookup_widget(GTK_WIDGET(button), wnames[i]);
-		if(w){
-			flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-		}//if
+		w=glade_xml_get_widget(glade, wnames[i]);
+		if(w){ flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)); }//if
 	}//for
 
-	w=lookup_widget(GTK_WIDGET(button), wnames[2]);
 #ifdef HILDON
-	flags[2] = hildon_number_editor_get_value(HILDON_NUMBER_EDITOR(w));
+	flags[2] = get_hildon_value(wnames[2]);
 #else
+	w=glade_xml_get_widget(glade, wnames[2]);
 	flags[2] = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
 #endif
 
 	romname = gtk_combo_box_get_active_text(GTK_COMBO_BOX(romlist));
 
 #ifdef HILDON
-	hildon_banner_show_information(mainwindow, NULL, _("Launching vba"));
+	hildon_banner_show_information(GTK_WIDGET(hildonwindow), NULL, _("Launching vba"));
 #endif
 
 	status = launch_vba(romname, FALSE, flags[0], flags[1], flags[2]);
@@ -159,30 +159,28 @@ void on_b_ok3_clicked                   (GtkButton       *button,
 
 
 // NeoGeo
-void on_b_ok4_clicked                   (GtkButton       *button,
+void on_ng_launch_clicked                   (GtkButton       *button,
                                         gpointer         user_data) {
 	gchar	*romname = NULL,
 		*cursor = NULL,
 		*biospath = NULL;
 	GtkWidget	*w = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist4"),
-			*mainwindow = lookup_widget(GTK_WIDGET(button), "mainwindow");
-	const char*	wnames[2] = { "cb_sound4", "cb_scale4" };
+			*romlist = glade_xml_get_widget(glade, "ng_romlist"),
+			*mainwindow = glade_xml_get_widget(glade, "mainwindow");
+	const char*	wnames[2] = { "ng_sound", "ng_double" };
 	gint	flags[3] = { TRUE, TRUE };
 	gint	i = 0,
 		status = 0;
 
 	for(i=0;i<2;++i){
-		w=lookup_widget(GTK_WIDGET(button), wnames[i]);
-		if(w){
-			flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-		}//if
+		w=glade_xml_get_widget(glade, wnames[i]);
+		if(w){ flags[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)); }//if
 	}//for
 
 	romname = gtk_combo_box_get_active_text(GTK_COMBO_BOX(romlist));
 
 #ifdef HILDON
-	hildon_banner_show_information(mainwindow, NULL, _("Launching gngeo"));
+	hildon_banner_show_information(GTK_WIDGET(hildonwindow), NULL, _("Launching gngeo"));
 #endif
 
 	status = launch_gngeo(romname, biospath, flags[0], flags[1]);
@@ -199,10 +197,10 @@ void on_b_ok4_clicked                   (GtkButton       *button,
 
 
 // MAME
-void on_file_browse_clicked(GtkButton *button, gpointer user_data) {
+void on_mame_browse_clicked(GtkButton *button, gpointer user_data) {
 	GtkWidget	*filedialog = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist"),
-			*parent = lookup_widget(GTK_WIDGET(button), "mainwindow");
+			*romlist = glade_xml_get_widget(glade, "mame_romlist"),
+			*parent = glade_xml_get_widget(glade, "mainwindow");
 	GList	*cursor = NULL;
 
 #ifdef HILDON
@@ -233,10 +231,10 @@ void on_file_browse_clicked(GtkButton *button, gpointer user_data) {
 
 
 // NES
-void on_file_browse2_clicked(GtkButton *button, gpointer user_data) {
+void on_nes_browse_clicked(GtkButton *button, gpointer user_data) {
 	GtkWidget	*filedialog = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist2"),
-			*parent = lookup_widget(GTK_WIDGET(button), "mainwindow");
+			*romlist = glade_xml_get_widget(glade, "nes_romlist"),
+			*parent = glade_xml_get_widget(glade, "mainwindow");
 	GList	*cursor = NULL;
 
 #ifdef HILDON
@@ -266,10 +264,10 @@ void on_file_browse2_clicked(GtkButton *button, gpointer user_data) {
 
 
 // GBA
-void on_file_browse3_clicked(GtkButton *button, gpointer user_data) {
+void on_gba_browse_clicked(GtkButton *button, gpointer user_data) {
 	GtkWidget	*filedialog = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist3"),
-			*parent = lookup_widget(GTK_WIDGET(button), "mainwindow");
+			*romlist = glade_xml_get_widget(glade, "gba_romlist"),
+			*parent = glade_xml_get_widget(glade, "mainwindow");
 	GList	*cursor = NULL;
 
 #ifdef HILDON
@@ -299,10 +297,10 @@ void on_file_browse3_clicked(GtkButton *button, gpointer user_data) {
 
 
 // NeoGeo
-void on_file_browse4_clicked(GtkButton *button, gpointer user_data) {
+void on_ng_browse_clicked(GtkButton *button, gpointer user_data) {
 	GtkWidget	*filedialog = NULL,
-			*romlist = lookup_widget(GTK_WIDGET(button), "romlist4"),
-			*parent = lookup_widget(GTK_WIDGET(button), "mainwindow");
+			*romlist = glade_xml_get_widget(glade, "ng_romlist"),
+			*parent = glade_xml_get_widget(glade, "mainwindow");
 	GList	*cursor = NULL;
 
 #ifdef HILDON
@@ -333,13 +331,13 @@ void on_file_browse4_clicked(GtkButton *button, gpointer user_data) {
 
 
 // NES Sound toggle clicked
-void on_cb_sound2_clicked(GtkToggleButton *button, gpointer user_data) {
-	GtkWidget	*spinner= lookup_widget(GTK_WIDGET(button), "frameskip");
+void on_nes_sound_toggled(GtkToggleButton *button, gpointer user_data) {
 	gboolean	checked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
 
 #ifdef HILDON
-	gint		fsvalue = hildon_number_editor_get_value(HILDON_NUMBER_EDITOR(spinner));
+	gint		fsvalue = get_hildon_value("nes_frameskip");
 #else
+	GtkWidget	*spinner= glade_xml_get_widget(glade, "nes_frameskip");
 	gint		fsvalue = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinner));
 #endif
 
@@ -350,16 +348,16 @@ void on_cb_sound2_clicked(GtkToggleButton *button, gpointer user_data) {
 	}
 
 #ifdef HILDON
-	hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(spinner), fsvalue);
+	set_hildon_value("nes_frameskip", fsvalue);
 #else
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner), fsvalue);
 #endif
 }//on_cb_sound2_clicked
 
-#ifdef HILDON
 gboolean key_press_cb(GtkWidget * widget, GdkEventKey * event,
-		                      HildonWindow * window)
+		                      gpointer window)
 {
+#ifdef HILDON
 	gint	width = 0,
 		height = 0;
 
@@ -376,16 +374,56 @@ gboolean key_press_cb(GtkWidget * widget, GdkEventKey * event,
 			break;
 	}//switch on key pressed
 
+#endif
 	return FALSE;
 }
-#endif
 
 void show_error_dialog(GtkWidget *mainwindow, const char *error_msg){
 #ifdef HILDON
-		hildon_banner_show_information(mainwindow, NULL, error_msg);
+		hildon_banner_show_information(GTK_WIDGET(hildonwindow), NULL, error_msg);
 #else
 		GtkWidget *msgdialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "Error: %s",error_msg);
 		gtk_dialog_run(GTK_DIALOG(msgdialog));
 		gtk_widget_destroy(msgdialog);
 #endif
 }//show_error_dialog
+
+
+
+#ifdef HILDON
+HildonNumberEditor* get_ned(const char *spin_name) {
+	GtkWidget	*w = glade_xml_get_widget(glade, spin_name);
+	gchar *wname = g_strdup_printf("%s%s", spin_name, "_hildon");
+	GList *list = gtk_container_get_children(GTK_CONTAINER(gtk_widget_get_ancestor(w, GTK_TYPE_HBOX))),
+	      *cursor = NULL;
+
+	if(cursor = g_list_find_custom(list, wname, check_name)) {
+		w = GTK_WIDGET(cursor->data);
+	}//if found
+	
+	g_list_free(list);
+	g_free(wname);
+
+	return HILDON_NUMBER_EDITOR(w);
+}//get_ned
+
+
+gint get_hildon_value(const char *spin_name) {
+	HildonNumberEditor *ned = get_ned(spin_name);
+	return ned? hildon_number_editor_get_value(ned): 0;
+}//get_hildon_value
+
+
+void set_hildon_value(const char *spin_name, gint val){
+	HildonNumberEditor *ned = get_ned(spin_name);
+	if(ned){ hildon_number_editor_set_value(ned, val); }
+}//set_hildon_value
+
+
+gint check_name(gconstpointer widget, gconstpointer name) {
+	return widget?
+		strcmp(gtk_widget_get_name(GTK_WIDGET(widget)), (char*)name):
+		1;
+}//check_name
+#endif
+
